# -*- coding: latin-1 -*-
#    QUENLIG: Questionnaire en ligne (Online interactive tutorial)
#    Copyright (C) 2005-2006 Thierry EXCOFFIER, Universite Claude Bernard
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#    Contact: Thierry.EXCOFFIER@bat710.univ-lyon1.fr
#

from questions import *
from check import *

cut_required = require("cut",
                       """Comme vous le savez dj la commande pour
                       extraire des colonnes d'un fichier est <tt>cut</tt>""")

ordre = """Il est plus logique d'indiquer le sparateur avant
de spcifier la colonne que l'on veut"""

add(name="extraire",
    required=["manuel:chercher"],
    before="""De nombreux fichiers ou rsultats de commandes
    sont organiss sous la forme d'un tableau de donne avec
    une ligne par donne et des champs dlimits par un
    sparateur.
    <p>
    Par exemple le fichier  <tt>/etc/passwd</tt>
    """,
    question="""Quel est le nom de la commande permettant
    d'extraire une colonne (ou plusieurs) d'un fichier&nbsp;?""",
    tests=(
    bad("awk",
        """Cette commande permet de le faire, mais cela ncessite
        l'criture d'un programme."""),
    good("cut"),
    ),
    indices=("C'est le verbe 'couper' en anglais",
             "C'est en trois lettres",
             "C'est le mot employ dans l'expression 'couper/coller'",
             ),
    )

add(name="les shells",
    required=["extraire", "manuel:section commande"],
    question="""Commande permettant d'extraire du fichier
    <tt>/etc/passwd</tt> seulement
    la colonne contenant le nom du programme (le shell) qui
    est lanc au moment de la connexion.""",
    tests=(
    shell_good("cut -d: -f7 /etc/passwd"),
    shell_good("cut -f7 -d: /etc/passwd", ordre),
    cut_required,
    reject("<", "Ne faites pas de redirections inutiles"),
    shell_require("<argument>-d:</argument>",
                  "Vous devez indiquer que ':' est le dlimiteur"),
    shell_require("<argument>-f7</argument>",
                  "Vous devez indiquer la colonne que vous voulez extraire"),
    shell_require("<argument>/etc/passwd</argument>",
                  """Vous devez indiquer le nom du fichier
                  ou extraire la colonne"""),
    shell_display,
    ),
    )

add(name="utilisateurs",
    required=["les shells", "sh:remplacement"],
    question="""Commande permettant de stocker
    dans la variable <tt>A</tt> la liste des noms
    d'utilisateurs dfinis dans <tt>/etc/passwd</tt>.""",
    tests=(
    reject("<", "Ne faites pas de redirections inutiles"),
    require("1", "Vous n'indiquez pas que la colonne des utilisateurs est la premire"),
    require("$", """Les prrequis vous indiquent
    qu'il faut faire un remplacement"""),
    shell_good((
    "A=$(cut -d: -f1 /etc/passwd)",
    'A="$(cut -d: -f1 /etc/passwd)"'
                ) ),
    shell_good((
    "A=$(cut -f1 -d: /etc/passwd)",
    'A="$(cut -f1 -d: /etc/passwd)"'), ordre),
    shell_display,
    ),
    )

import remplacer

add(name="espaces multiples",
    required=["les shells", "remplacer:intro", "pipeline:intro"],
    question="""Donnez la ligne de commande permettant d'extraire
    la cinquime colonne de l'entre standard sachant que
    les colonnes sont spares par de multiples espaces et non un seul.""",
    tests=(
    require('sed', "On utilise <tt>sed</tt> pour faire le remplacement"),
    require('/g', "Il y a plusieurs substitutions  faire, pas une seule"),
    cut_required,
    reject('/ */',
           """Votre expression rgulire remplace un espace par rien.
           Les colonnes vont donc disparatre"""),
    reject('[ ]', " quoi servent les crochets autour de l'espace&nbsp;?"),
    
    shell_good("sed 's/  */ /g' | cut -d' ' -f5",
               dumb_replace=remplacer.dumb_replace),
    shell_good("sed -r 's/ +/ /g' | cut -d' ' -f5",
               dumb_replace=remplacer.dumb_replace),
    require(' *', "Il faut utiliser l'toile pour rpter l'espace."),
    require("5", "Vous n'avez pas indiqu le numro de la colonne  extraire"),
    shell_display,
    ),
    indices=(
    """Utilisez un filtre pour remplacer les suites de blancs
    par un seul blanc""",
    ),
    )
    

    
